@@ -16,7 +16,6 @@ running multiple build commands in parallel, confusion can occur).
 from __future__ import print_function
 import argparse
 import logging
-import os
 import sys
 
 from tools import shared
@@ -51,7 +50,6 @@ MINIMAL_TASKS = [
     'libsockets',
     'libc_rt_wasm',
     'struct_info',
-    'libc-wasm',
     'libstandalonewasm',
     'crt1',
     'libunwind-except'
@@ -173,27 +171,19 @@ def main():
     tasks = SYSTEM_TASKS + USER_TASKS
     auto_tasks = True
   if auto_tasks:
-    if shared.Settings.WASM_BACKEND:
-      skip_tasks = []
-      if shared.Settings.RELOCATABLE:
-        # we don't support PIC + pthreads yet
-        for task in SYSTEM_TASKS + USER_TASKS:
-          if '-mt' in task:
-            skip_tasks.append(task)
-          if 'pthread' in task and 'stub' not in task:
-            skip_tasks.append(task)
-        print('Skipping building of %s, because we don\'t support threads and PIC code.' % ', '.join(skip_tasks))
-      # cocos2d: must be ported, errors on
-      # "Cannot recognize the target platform; are you targeting an unsupported platform?"
-      skip_tasks += ['cocos2d']
-      tasks = [x for x in tasks if x not in skip_tasks]
-    else:
-      if os.environ.get('EMSCRIPTEN_NATIVE_OPTIMIZER'):
-        print('Skipping building of native-optimizer; EMSCRIPTEN_NATIVE_OPTIMIZER is environment.')
-      elif shared.EMSCRIPTEN_NATIVE_OPTIMIZER:
-        print('Skipping building of native-optimizer; EMSCRIPTEN_NATIVE_OPTIMIZER set in .emscripten config.')
-      else:
-        tasks += ['native_optimizer']
+    skip_tasks = []
+    if shared.Settings.RELOCATABLE:
+      # we don't support PIC + pthreads yet
+      for task in SYSTEM_TASKS + USER_TASKS:
+        if '-mt' in task:
+          skip_tasks.append(task)
+        if 'pthread' in task and 'stub' not in task:
+          skip_tasks.append(task)
+      print('Skipping building of %s, because we don\'t support threads and PIC code.' % ', '.join(skip_tasks))
+    # cocos2d: must be ported, errors on
+    # "Cannot recognize the target platform; are you targeting an unsupported platform?"
+    skip_tasks += ['cocos2d']
+    tasks = [x for x in tasks if x not in skip_tasks]
     print('Building targets: %s' % ' '.join(tasks))
   for what in tasks:
     logger.info('building and verifying ' + what)
