@@ -1,4 +1,4 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python3
 # Copyright 2013 The Emscripten Authors.  All rights reserved.
 # Emscripten is available under two separate licenses, the MIT license and the
 # University of Illinois/NCSA Open Source License.  Both these licenses can be
@@ -18,7 +18,7 @@ from tools.asm_module import AsmModule
 def run():
   try:
     me, main, side, out = sys.argv[:4]
-  except:
+  except ValueError:
     print('usage: emlink.py [main module] [side module] [output name]', file=sys.stderr)
     sys.exit(1)
 
