@@ -1,24 +1,13 @@
-// Copyright 2017 The Emscripten Authors.  All rights reserved.
-// Emscripten is available under two separate licenses, the MIT license and the
-// University of Illinois/NCSA Open Source License.  Both these licenses can be
-// found in the LICENSE file.
+/**
+ * @license
+ * Copyright 2017 The Emscripten Authors
+ * SPDX-License-Identifier: MIT
+ */
 
 // {{PREAMBLE_ADDITIONS}}
 
 var STACK_ALIGN = {{{ STACK_ALIGN }}};
 
-#if ASSERTIONS
-// stack management, and other functionality that is provided by the compiled code,
-// should not be used before it is ready
-stackSave = stackRestore = stackAlloc = function() {
-  abort('cannot use the stack before compiled code is ready to run, and has provided stack access');
-};
-
-function staticAlloc(size) {
-  abort('staticAlloc is no longer available at runtime; instead, perform static allocations at compile time (using makeStaticAlloc)');
-}
-#endif
-
 function dynamicAlloc(size) {
 #if ASSERTIONS
   assert(DYNAMICTOP_PTR);
@@ -28,13 +17,9 @@ function dynamicAlloc(size) {
 #endif
   var ret = HEAP32[DYNAMICTOP_PTR>>2];
   var end = (ret + size + 15) & -16;
-  if (end > _emscripten_get_heap_size()) {
 #if ASSERTIONS
-    abort('failure to dynamicAlloc - memory growth etc. is not supported there, call malloc/sbrk directly');
-#else
-    abort();
+  assert(end <= HEAP8.length, 'failure to dynamicAlloc - memory growth etc. is not supported there, call malloc/sbrk directly');
 #endif
-  }
   HEAP32[DYNAMICTOP_PTR>>2] = end;
   return ret;
 }
@@ -51,33 +36,6 @@ function warnOnce(text) {
   }
 }
 
-var asm2wasmImports = { // special asm2wasm imports
-    "f64-rem": function(x, y) {
-        return x % y;
-    },
-    "debugger": function() {
-        debugger;
-    }
-#if NEED_ALL_ASM2WASM_IMPORTS
-    ,
-    "f64-to-int": function(x) {
-        return x | 0;
-    },
-    "i32s-div": function(x, y) {
-        return ((x | 0) / (y | 0)) | 0;
-    },
-    "i32u-div": function(x, y) {
-        return ((x >>> 0) / (y >>> 0)) >>> 0;
-    },
-    "i32s-rem": function(x, y) {
-        return ((x | 0) % (y | 0)) | 0;
-    },
-    "i32u-rem": function(x, y) {
-        return ((x >>> 0) % (y >>> 0)) >>> 0;
-    }
-#endif // NEED_ALL_ASM2WASM_IMPORTS
-};
-
 #if RELOCATABLE
 // dynamic linker/loader (a-la ld.so on ELF systems)
 var LDSO = {
@@ -114,6 +72,11 @@ function fetchBinary(url) {
   });
 }
 
+function asmjsMangle(x) {
+  var unmangledSymbols = {{{ buildStringArray(WASM_FUNCTIONS_THAT_ARE_NOT_NAME_MANGLED) }}};
+  return x.indexOf('dynCall_') == 0 || unmangledSymbols.indexOf(x) != -1 ? x : '_' + x;
+}
+
 // loadDynamicLibrary loads dynamic library @ lib URL / path and returns handle for loaded DSO.
 //
 // Several flags affect the loading:
@@ -172,12 +135,12 @@ function loadDynamicLibrary(lib, flags) {
   LDSO.loadedLibNames[lib] = handle;
   LDSO.loadedLibs[handle] = dso;
 
-  // libData <- lib
-  function loadLibData() {
+  // libData <- libFile
+  function loadLibData(libFile) {
 #if WASM
     // for wasm, we can use fetch for async, but for fs mode we can only imitate it
     if (flags.fs) {
-      var libData = flags.fs.readFile(lib, {encoding: 'binary'});
+      var libData = flags.fs.readFile(libFile, {encoding: 'binary'});
       if (!(libData instanceof Uint8Array)) {
         libData = new Uint8Array(lib_data);
       }
@@ -185,17 +148,17 @@ function loadDynamicLibrary(lib, flags) {
     }
 
     if (flags.loadAsync) {
-      return fetchBinary(lib);
+      return fetchBinary(libFile);
     }
     // load the binary synchronously
-    return Module['readBinary'](lib);
+    return readBinary(libFile);
 #else
     // for js we only imitate async for both native & fs modes.
     var libData;
     if (flags.fs) {
-      libData = flags.fs.readFile(lib, {encoding: 'utf8'});
+      libData = flags.fs.readFile(libFile, {encoding: 'utf8'});
     } else {
-      libData = Module['read'](lib);
+      libData = read_(libFile);
     }
     return flags.loadAsync ? Promise.resolve(libData) : libData;
 #endif
@@ -206,7 +169,7 @@ function loadDynamicLibrary(lib, flags) {
 #if WASM
     return loadWebAssemblyModule(libData, flags)
 #else
-    var libModule = eval(libData)(
+    var libModule = /**@type{function(...)}*/(eval(libData))(
       alignFunctionTables(),
       Module
     );
@@ -261,21 +224,7 @@ function loadDynamicLibrary(lib, flags) {
       //
       // We should copy the symbols (which include methods and variables) from SIDE_MODULE to MAIN_MODULE.
 
-      var module_sym = sym;
-#if WASM_BACKEND
-      module_sym = '_' + sym;
-#else
-      // Module of SIDE_MODULE has not only the symbols (which should be copied)
-      // but also others (print*, asmGlobal*, FUNCTION_TABLE_**, NAMED_GLOBALS, and so on).
-      //
-      // When the symbol (which should be copied) is method, Module.* 's type becomes function.
-      // When the symbol (which should be copied) is variable, Module.* 's type becomes number.
-      // Except for the symbol prefix (_), there is no difference in the symbols (which should be copied) and others.
-      // So this just copies over compiled symbols (which start with _).
-      if (sym[0] !== '_') {
-        continue;
-      }
-#endif
+      var module_sym = asmjsMangle(sym);
 
       if (!Module.hasOwnProperty(module_sym)) {
         Module[module_sym] = libModule[sym];
@@ -312,6 +261,56 @@ function loadDynamicLibrary(lib, flags) {
 }
 
 #if WASM
+// Applies relocations to exported things.
+function relocateExports(exports, memoryBase, tableBase, moduleLocal) {
+  var relocated = {};
+
+  for (var e in exports) {
+    var value = exports[e];
+    if (typeof value === 'object') {
+      // a breaking change in the wasm spec, globals are now objects
+      // https://github.com/WebAssembly/mutable-global/issues/1
+      value = value.value;
+    }
+    if (typeof value === 'number') {
+      // relocate it - modules export the absolute value, they can't relocate before they export
+#if EMULATE_FUNCTION_POINTER_CASTS
+      // it may be a function pointer
+      if (e.substr(0, 3) == 'fp$' && typeof exports[e.substr(3)] === 'function') {
+        value += tableBase;
+      } else {
+#endif
+        value += memoryBase;
+#if EMULATE_FUNCTION_POINTER_CASTS
+      }
+#endif
+    }
+    relocated[e] = value;
+    if (moduleLocal) {
+      moduleLocal['_' + e] = value;
+    }
+  }
+  return relocated;
+}
+
+#if RELOCATABLE
+// Dynmamic version of shared.py:make_invoke.  This is needed for invokes
+// that originate from side modules since these are not known at JS
+// generation time.
+function createInvokeFunction(sig) {
+  return function() {
+    var sp = stackSave();
+    try {
+      return Module['dynCall_' + sig].apply(null, arguments);
+    } catch(e) {
+      stackRestore(sp);
+      if (e !== e+0 && e !== 'longjmp') throw e;
+      _setThrew(1, 0);
+    }
+  }
+}
+#endif
+
 // Loads a side module from binary data
 function loadWebAssemblyModule(binary, flags) {
   var int32View = new Uint32Array(new Uint8Array(binary.subarray(0, 24)).buffer);
@@ -369,12 +368,6 @@ function loadWebAssemblyModule(binary, flags) {
 #endif
     // prepare memory
     var memoryBase = alignMemory(getMemory(memorySize + memoryAlign), memoryAlign); // TODO: add to cleanups
-    // The static area consists of explicitly initialized data, followed by zero-initialized data.
-    // The latter may need zeroing out if the MAIN_MODULE has already used this memory area before
-    // dlopen'ing the SIDE_MODULE.  Since we don't know the size of the explicitly initialized data
-    // here, we just zero the whole thing, which is suboptimal, but should at least resolve bugs
-    // from uninitialized memory.
-    for (var i = memoryBase; i < memoryBase + memorySize; ++i) HEAP8[i] = 0;
     // prepare env imports
     var env = asmLibraryArg;
     // TODO: use only __memory_base and __table_base, need to update asm.js backend
@@ -384,7 +377,11 @@ function loadWebAssemblyModule(binary, flags) {
     table.grow(tableSize);
     assert(table === originalTable);
     // zero-initialize memory and table
-    // TODO: in some cases we can tell it is already zero initialized
+    // The static area consists of explicitly initialized data, followed by zero-initialized data.
+    // The latter may need zeroing out if the MAIN_MODULE has already used this memory area before
+    // dlopen'ing the SIDE_MODULE.  Since we don't know the size of the explicitly initialized data
+    // here, we just zero the whole thing, which is suboptimal, but should at least resolve bugs
+    // from uninitialized memory.
     for (var i = memoryBase; i < memoryBase + memorySize; i++) {
       HEAP8[i] = 0;
     }
@@ -398,16 +395,31 @@ function loadWebAssemblyModule(binary, flags) {
     // b) Symbols from loaded modules are not always added to the global Module.
     var moduleLocal = {};
 
-    var resolveSymbol = function(sym, type) {
-#if WASM_BACKEND
-      sym = '_' + sym;
+    var resolveSymbol = function(sym, type, legalized) {
+#if WASM_BIGINT
+      assert(!legalized);
+#else
+      if (legalized) {
+        sym = 'orig$' + sym;
+      }
+#endif
+
+      var resolved = Module["asm"][sym];
+      if (!resolved) {
+        var mangled = asmjsMangle(sym);
+        resolved = Module[mangled];
+        if (!resolved) {
+          resolved = moduleLocal[mangled];
+        }
+#if RELOCATABLE
+        if (!resolved && sym.startsWith('invoke_')) {
+          resolved = createInvokeFunction(sym.split('_')[1]);
+        }
 #endif
-      var resolved = Module[sym];
-      if (!resolved)
-        resolved = moduleLocal[sym];
 #if ASSERTIONS
-      assert(resolved, 'missing linked ' + type + ' `' + sym + '`. perhaps a side module was not linked in? if this global was expected to arrive from a system library, try to build the MAIN_MODULE with EMCC_FORCE_STDLIBS=1 in the environment');
+        assert(resolved, 'missing linked ' + type + ' `' + sym + '`. perhaps a side module was not linked in? if this global was expected to arrive from a system library, try to build the MAIN_MODULE with EMCC_FORCE_STDLIBS=1 in the environment');
 #endif
+      }
       return resolved;
     }
 
@@ -457,36 +469,35 @@ function loadWebAssemblyModule(binary, flags) {
           assert(parts.length == 3)
           var name = parts[1];
           var sig = parts[2];
+#if WASM_BIGINT
+          var legalized = false;
+#else
+          var legalized = sig.indexOf('j') >= 0; // check for i64s
+#endif
           var fp = 0;
           return obj[prop] = function() {
             if (!fp) {
-              var f = resolveSymbol(name, 'function');
-              fp = addFunctionWasm(f, sig);
+              var f = resolveSymbol(name, 'function', legalized);
+              fp = addFunction(f, sig);
             }
             return fp;
           };
         }
-        if (prop.startsWith('invoke_')) {
-          // A missing invoke, i.e., an invoke for a function type
-          // present in the dynamic library but not in the main JS,
-          // and the dynamic library cannot provide JS for it. Use
-          // the generic "X" invoke for it.
-          return obj[prop] = invoke_X;
-        }
         // otherwise this is regular function import - call it indirectly
         return obj[prop] = function() {
           return resolveSymbol(prop, 'function').apply(null, arguments);
         };
       }
     };
+    var proxy = new Proxy(env, proxyHandler);
     var info = {
       global: {
         'NaN': NaN,
         'Infinity': Infinity,
       },
       'global.Math': Math,
-      env: new Proxy(env, proxyHandler),
-      'asm2wasm': asm2wasmImports
+      env: proxy,
+      {{{ WASI_MODULE_NAME }}}: proxy,
     };
 #if ASSERTIONS
     var oldTable = [];
@@ -496,7 +507,6 @@ function loadWebAssemblyModule(binary, flags) {
 #endif
 
     function postInstantiation(instance, moduleLocal) {
-      var exports = {};
 #if ASSERTIONS
       // the table should be unchanged
       assert(table === originalTable);
@@ -510,33 +520,7 @@ function loadWebAssemblyModule(binary, flags) {
         assert(table.get(tableBase + i) !== undefined, 'table entry was not filled in');
       }
 #endif
-      for (var e in instance.exports) {
-        var value = instance.exports[e];
-        if (typeof value === 'object') {
-          // a breaking change in the wasm spec, globals are now objects
-          // https://github.com/WebAssembly/mutable-global/issues/1
-          value = value.value;
-        }
-        if (typeof value === 'number') {
-          // relocate it - modules export the absolute value, they can't relocate before they export
-#if EMULATE_FUNCTION_POINTER_CASTS
-          // it may be a function pointer
-          if (e.substr(0, 3) == 'fp$' && typeof instance.exports[e.substr(3)] === 'function') {
-            value = value + tableBase;
-          } else {
-#endif
-            value = value + memoryBase;
-#if EMULATE_FUNCTION_POINTER_CASTS
-          }
-#endif
-        }
-        exports[e] = value;
-#if WASM_BACKEND
-        moduleLocal['_' + e] = value;
-#else
-        moduleLocal[e] = value;
-#endif
-      }
+      var exports = relocateExports(instance.exports, memoryBase, tableBase, moduleLocal);
       // initialize the module
       var init = exports['__post_instantiate'];
       if (init) {
@@ -579,239 +563,7 @@ Module['loadWebAssemblyModule'] = loadWebAssemblyModule;
 #endif // WASM
 #endif // RELOCATABLE
 
-#if EMULATED_FUNCTION_POINTERS
-#if WASM == 0
-function getFunctionTables(module) {
-  if (!module) module = Module;
-  var tables = {};
-  for (var t in module) {
-    if (/^FUNCTION_TABLE_.*/.test(t)) {
-      var table = module[t];
-      if (typeof table === 'object') tables[t.substr('FUNCTION_TABLE_'.length)] = table;
-    }
-  }
-  return tables;
-}
-
-function alignFunctionTables(module) {
-  var tables = getFunctionTables(module);
-  var maxx = 0;
-  for (var sig in tables) {
-    maxx = Math.max(maxx, tables[sig].length);
-  }
-  assert(maxx >= 0);
-  for (var sig in tables) {
-    var table = tables[sig];
-    while (table.length < maxx) table.push(0);
-  }
-  return maxx;
-}
-#endif // WASM == 0
-
-#if RELOCATABLE
-// register functions from a new module being loaded
-function registerFunctions(sigs, newModule) {
-  sigs.forEach(function(sig) {
-    if (!Module['FUNCTION_TABLE_' + sig]) {
-      Module['FUNCTION_TABLE_' + sig] = [];
-    }
-  });
-  var oldMaxx = alignFunctionTables(); // align the new tables we may have just added
-  var newMaxx = alignFunctionTables(newModule);
-  var maxx = oldMaxx + newMaxx;
-  sigs.forEach(function(sig) {
-    var newTable = newModule['FUNCTION_TABLE_' + sig];
-    var oldTable = Module['FUNCTION_TABLE_' + sig];
-    assert(newTable !== oldTable);
-    assert(oldTable.length === oldMaxx);
-    for (var i = 0; i < newTable.length; i++) {
-      oldTable.push(newTable[i]);
-    }
-    assert(oldTable.length === maxx);
-  });
-  assert(maxx === alignFunctionTables()); // align the ones we didn't touch
-}
-// export this so side modules can use it
-Module['registerFunctions'] = registerFunctions;
-#endif // RELOCATABLE
-#endif // EMULATED_FUNCTION_POINTERS
-
-#if !WASM_BACKEND && EMULATED_FUNCTION_POINTERS == 0
-var jsCallStartIndex = 1;
-var functionPointers = new Array({{{ RESERVED_FUNCTION_POINTERS }}});
-#endif // !WASM_BACKEND && EMULATED_FUNCTION_POINTERS == 0
-
-#if WASM
-// Wraps a JS function as a wasm function with a given signature.
-// In the future, we may get a WebAssembly.Function constructor. Until then,
-// we create a wasm module that takes the JS function as an import with a given
-// signature, and re-exports that as a wasm function.
-function convertJsFunctionToWasm(func, sig) {
-#if WASM2JS
-  return func;
-#endif
-
-  // The module is static, with the exception of the type section, which is
-  // generated based on the signature passed in.
-  var typeSection = [
-    0x01, // id: section,
-    0x00, // length: 0 (placeholder)
-    0x01, // count: 1
-    0x60, // form: func
-  ];
-  var sigRet = sig.slice(0, 1);
-  var sigParam = sig.slice(1);
-  var typeCodes = {
-    'i': 0x7f, // i32
-    'j': 0x7e, // i64
-    'f': 0x7d, // f32
-    'd': 0x7c, // f64
-  };
-
-  // Parameters, length + signatures
-  typeSection.push(sigParam.length);
-  for (var i = 0; i < sigParam.length; ++i) {
-    typeSection.push(typeCodes[sigParam[i]]);
-  }
-
-  // Return values, length + signatures
-  // With no multi-return in MVP, either 0 (void) or 1 (anything else)
-  if (sigRet == 'v') {
-    typeSection.push(0x00);
-  } else {
-    typeSection = typeSection.concat([0x01, typeCodes[sigRet]]);
-  }
-
-  // Write the overall length of the type section back into the section header
-  // (excepting the 2 bytes for the section id and length)
-  typeSection[1] = typeSection.length - 2;
-
-  // Rest of the module is static
-  var bytes = new Uint8Array([
-    0x00, 0x61, 0x73, 0x6d, // magic ("\0asm")
-    0x01, 0x00, 0x00, 0x00, // version: 1
-  ].concat(typeSection, [
-    0x02, 0x07, // import section
-      // (import "e" "f" (func 0 (type 0)))
-      0x01, 0x01, 0x65, 0x01, 0x66, 0x00, 0x00,
-    0x07, 0x05, // export section
-      // (export "f" (func 0 (type 0)))
-      0x01, 0x01, 0x66, 0x00, 0x00,
-  ]));
-
-   // We can compile this wasm module synchronously because it is very small.
-  // This accepts an import (at "e.f"), that it reroutes to an export (at "f")
-  var module = new WebAssembly.Module(bytes);
-  var instance = new WebAssembly.Instance(module, {
-    e: {
-      f: func
-    }
-  });
-  var wrappedFunc = instance.exports.f;
-  return wrappedFunc;
-}
-
-// Add a wasm function to the table.
-function addFunctionWasm(func, sig) {
-  var table = wasmTable;
-  var ret = table.length;
-
-  // Grow the table
-  try {
-    table.grow(1);
-  } catch (err) {
-    if (!err instanceof RangeError) {
-      throw err;
-    }
-    throw 'Unable to grow wasm table. Use a higher value for RESERVED_FUNCTION_POINTERS or set ALLOW_TABLE_GROWTH.';
-  }
-
-  // Insert new element
-  try {
-    // Attempting to call this with JS function will cause of table.set() to fail
-    table.set(ret, func);
-  } catch (err) {
-    if (!err instanceof TypeError) {
-      throw err;
-    }
-    assert(typeof sig !== 'undefined', 'Missing signature argument to addFunction');
-    var wrapped = convertJsFunctionToWasm(func, sig);
-    table.set(ret, wrapped);
-  }
-
-  return ret;
-}
-
-function removeFunctionWasm(index) {
-  // TODO(sbc): Look into implementing this to allow re-using of table slots
-}
-#endif
-
-// 'sig' parameter is required for the llvm backend but only when func is not
-// already a WebAssembly function.
-function addFunction(func, sig) {
-#if ASSERTIONS == 2
-  if (typeof sig === 'undefined') {
-    err('warning: addFunction(): You should provide a wasm function signature string as a second argument. This is not necessary for asm.js and asm2wasm, but can be required for the LLVM wasm backend, so it is recommended for full portability.');
-  }
-#endif // ASSERTIONS
-
-#if WASM_BACKEND
-  return addFunctionWasm(func, sig);
-#else
-
-#if EMULATED_FUNCTION_POINTERS == 0
-  var base = 0;
-  for (var i = base; i < base + {{{ RESERVED_FUNCTION_POINTERS }}}; i++) {
-    if (!functionPointers[i]) {
-      functionPointers[i] = func;
-      return jsCallStartIndex + i;
-    }
-  }
-  throw 'Finished up all reserved function pointers. Use a higher value for RESERVED_FUNCTION_POINTERS.';
-
-#else // EMULATED_FUNCTION_POINTERS == 0
-
-#if WASM
-  return addFunctionWasm(func, sig);
-#else
-  alignFunctionTables(); // TODO: we should rely on this being an invariant
-  var tables = getFunctionTables();
-  var ret = -1;
-  for (var sig in tables) {
-    var table = tables[sig];
-    if (ret < 0) ret = table.length;
-    else assert(ret === table.length);
-    table.push(func);
-  }
-  return ret;
-#endif // WASM
-
-#endif // EMULATED_FUNCTION_POINTERS == 0
-#endif // WASM_BACKEND
-}
-
-function removeFunction(index) {
-#if WASM_BACKEND
-  removeFunctionWasm(index);
-#else
-
-#if EMULATED_FUNCTION_POINTERS == 0
-  functionPointers[index-jsCallStartIndex] = null;
-#else
-#if WASM
-  removeFunctionWasm(index);
-#else
-  alignFunctionTables(); // XXX we should rely on this being an invariant
-  var tables = getFunctionTables();
-  for (var sig in tables) {
-    tables[sig][index] = null;
-  }
-#endif // WASM
-
-#endif // EMULATE_FUNCTION_POINTER_CASTS == 0
-#endif // WASM_BACKEND
-}
+#include "runtime_functions.js"
 
 var funcWrappers = {};
 
@@ -842,66 +594,18 @@ function getFuncWrapper(func, sig) {
   return sigCache[func];
 }
 
-#if RUNTIME_DEBUG
-var runtimeDebug = true; // Switch to false at runtime to disable logging at the right times
-
-var printObjectList = [];
-
-function prettyPrint(arg) {
-  if (typeof arg == 'undefined') return '!UNDEFINED!';
-  if (typeof arg == 'boolean') arg = arg + 0;
-  if (!arg) return arg;
-  var index = printObjectList.indexOf(arg);
-  if (index >= 0) return '<' + arg + '|' + index + '>';
-  if (arg.toString() == '[object HTMLImageElement]') {
-    return arg + '\n\n';
-  }
-  if (arg.byteLength) {
-    return '{' + Array.prototype.slice.call(arg, 0, Math.min(arg.length, 400)) + '}'; // Useful for correct arrays, less so for compiled arrays, see the code below for that
-    var buf = new ArrayBuffer(32);
-    var i8buf = new Int8Array(buf);
-    var i16buf = new Int16Array(buf);
-    var f32buf = new Float32Array(buf);
-    switch(arg.toString()) {
-      case '[object Uint8Array]':
-        i8buf.set(arg.subarray(0, 32));
-        break;
-      case '[object Float32Array]':
-        f32buf.set(arg.subarray(0, 5));
-        break;
-      case '[object Uint16Array]':
-        i16buf.set(arg.subarray(0, 16));
-        break;
-      default:
-        alert('unknown array for debugging: ' + arg);
-        throw 'see alert';
-    }
-    var ret = '{' + arg.byteLength + ':\n';
-    var arr = Array.prototype.slice.call(i8buf);
-    ret += 'i8:' + arr.toString().replace(/,/g, ',') + '\n';
-    arr = Array.prototype.slice.call(f32buf, 0, 8);
-    ret += 'f32:' + arr.toString().replace(/,/g, ',') + '}';
-    return ret;
-  }
-  if (typeof arg == 'object') {
-    printObjectList.push(arg);
-    return '<' + arg + '|' + (printObjectList.length-1) + '>';
-  }
-  if (typeof arg == 'number') {
-    if (arg > 0) return '0x' + arg.toString(16) + ' (' + arg + ')';
-  }
-  return arg;
-}
-#endif
+#include "runtime_debug.js"
 
 function makeBigInt(low, high, unsigned) {
   return unsigned ? ((+((low>>>0)))+((+((high>>>0)))*4294967296.0)) : ((+((low>>>0)))+((+((high|0)))*4294967296.0));
 }
 
+/** @param {Array=} args */
 function dynCall(sig, ptr, args) {
   if (args && args.length) {
 #if ASSERTIONS
-    assert(args.length == sig.length-1);
+    // j (64-bit integer) must be passed in as two numbers [low 32, high 32].
+    assert(args.length === sig.substring(1).replace(/j/g, '--').length);
 #endif
 #if ASSERTIONS
     assert(('dynCall_' + sig) in Module, 'bad function pointer type - no table for sig \'' + sig + '\'');
@@ -922,11 +626,11 @@ var tempRet0 = 0;
 
 var setTempRet0 = function(value) {
   tempRet0 = value;
-}
+};
 
 var getTempRet0 = function() {
   return tempRet0;
-}
+};
 
 #if RETAIN_COMPILER_SETTINGS
 var compilerSettings = {{{ JSON.stringify(makeRetainedCompilerSettings()) }}} ;
@@ -943,15 +647,6 @@ function getCompilerSetting(name) {
 #endif // ASSERTIONS
 #endif // RETAIN_COMPILER_SETTINGS
 
-var Runtime = {
-#if ASSERTIONS
-  // helpful errors
-  getTempRet0: function() { abort('getTempRet0() is now a top-level function, after removing the Runtime object. Remove "Runtime."') },
-  staticAlloc: function() { abort('staticAlloc() is now a top-level function, after removing the Runtime object. Remove "Runtime."') },
-  stackAlloc: function() { abort('stackAlloc() is now a top-level function, after removing the Runtime object. Remove "Runtime."') },
-#endif
-};
-
 // The address globals begin at. Very low in memory, for code size and optimization opportunities.
 // Above 0 is static memory, starting with globals.
 // Then the stack.
@@ -962,17 +657,10 @@ var GLOBAL_BASE = {{{ GLOBAL_BASE }}};
 GLOBAL_BASE = alignMemory(GLOBAL_BASE, {{{ MAX_GLOBAL_ALIGN || 1 }}});
 #endif
 
-#if WASM_BACKEND && USE_PTHREADS
-// The wasm backend path does not have a way to set the stack max, so we can
-// just implement this function in a trivial way
-function establishStackSpace(base, max) {
-  stackRestore(max);
-}
-
+#if USE_PTHREADS
 // JS library code refers to Atomics in the manner used from asm.js, provide
 // the same API here.
 var Atomics_load = Atomics.load;
 var Atomics_store = Atomics.store;
 var Atomics_compareExchange = Atomics.compareExchange;
 #endif
-
