@@ -5,8 +5,7 @@
  * found in the LICENSE file.
  */
 
-#ifndef __emscripten_events_h__
-#define __emscripten_events_h__
+#pragma once
 
 #ifdef __cplusplus
 #if !defined(__DEFINED_pthread_t)
@@ -76,7 +75,6 @@ extern "C" {
 
 #define EMSCRIPTEN_RESULT int
 
-
 #define EMSCRIPTEN_RESULT_SUCCESS              0
 #define EMSCRIPTEN_RESULT_DEFERRED             1
 #define EMSCRIPTEN_RESULT_NOT_SUPPORTED       -1
@@ -88,6 +86,11 @@ extern "C" {
 #define EMSCRIPTEN_RESULT_NO_DATA             -7
 #define EMSCRIPTEN_RESULT_TIMED_OUT           -8
 
+#define EMSCRIPTEN_EVENT_TARGET_INVALID        0
+#define EMSCRIPTEN_EVENT_TARGET_DOCUMENT       ((const char*)1)
+#define EMSCRIPTEN_EVENT_TARGET_WINDOW         ((const char*)2)
+#define EMSCRIPTEN_EVENT_TARGET_SCREEN         ((const char*)3)
+
 #define EM_BOOL int
 #define EM_TRUE 1
 #define EM_FALSE 0
@@ -104,19 +107,19 @@ extern "C" {
 #define EM_HTML5_LONG_STRING_LEN_BYTES 128
 
 typedef struct EmscriptenKeyboardEvent {
-  EM_UTF8 key[EM_HTML5_SHORT_STRING_LEN_BYTES];
-  EM_UTF8 code[EM_HTML5_SHORT_STRING_LEN_BYTES];
   unsigned long location;
   EM_BOOL ctrlKey;
   EM_BOOL shiftKey;
   EM_BOOL altKey;
   EM_BOOL metaKey;
   EM_BOOL repeat;
-  EM_UTF8 locale[EM_HTML5_SHORT_STRING_LEN_BYTES];
-  EM_UTF8 charValue[EM_HTML5_SHORT_STRING_LEN_BYTES];
   unsigned long charCode;
   unsigned long keyCode;
   unsigned long which;
+  EM_UTF8 key[EM_HTML5_SHORT_STRING_LEN_BYTES];
+  EM_UTF8 code[EM_HTML5_SHORT_STRING_LEN_BYTES];
+  EM_UTF8 charValue[EM_HTML5_SHORT_STRING_LEN_BYTES];
+  EM_UTF8 locale[EM_HTML5_SHORT_STRING_LEN_BYTES];
 } EmscriptenKeyboardEvent;
 
 
@@ -126,7 +129,6 @@ extern EMSCRIPTEN_RESULT emscripten_set_keydown_callback_on_thread(const char *t
 extern EMSCRIPTEN_RESULT emscripten_set_keyup_callback_on_thread(const char *target, void *userData, EM_BOOL useCapture, em_key_callback_func callback, pthread_t targetThread);
 
 typedef struct EmscriptenMouseEvent {
-  double timestamp;
   long screenX;
   long screenY;
   long clientX;
@@ -141,6 +143,7 @@ typedef struct EmscriptenMouseEvent {
   long movementY;
   long targetX;
   long targetY;
+  // canvasX and canvasY are deprecated - there no longer exists a Module['canvas'] object, so canvasX/Y are no longer reported (register a listener on canvas directly to get canvas coordinates, or translate manually)
   long canvasX;
   long canvasY;
   long padding;
@@ -205,7 +208,6 @@ extern EMSCRIPTEN_RESULT emscripten_set_focusin_callback_on_thread(const char *t
 extern EMSCRIPTEN_RESULT emscripten_set_focusout_callback_on_thread(const char *target, void *userData, EM_BOOL useCapture, em_focus_callback_func callback, pthread_t targetThread);
 
 typedef struct EmscriptenDeviceOrientationEvent {
-  double timestamp;
   double alpha;
   double beta;
   double gamma;
@@ -218,8 +220,11 @@ extern EMSCRIPTEN_RESULT emscripten_set_deviceorientation_callback_on_thread(voi
 
 extern EMSCRIPTEN_RESULT emscripten_get_deviceorientation_status(EmscriptenDeviceOrientationEvent *orientationState);
 
+#define EMSCRIPTEN_DEVICE_MOTION_EVENT_SUPPORTS_ACCELERATION                   0x01
+#define EMSCRIPTEN_DEVICE_MOTION_EVENT_SUPPORTS_ACCELERATION_INCLUDING_GRAVITY 0x02
+#define EMSCRIPTEN_DEVICE_MOTION_EVENT_SUPPORTS_ROTATION_RATE                  0x04
+
 typedef struct EmscriptenDeviceMotionEvent {
-  double timestamp;
   double accelerationX;
   double accelerationY;
   double accelerationZ;
@@ -229,6 +234,7 @@ typedef struct EmscriptenDeviceMotionEvent {
   double rotationRateAlpha;
   double rotationRateBeta;
   double rotationRateGamma;
+  int supportedFields;
 } EmscriptenDeviceMotionEvent;
 
 
@@ -356,6 +362,7 @@ typedef struct EmscriptenTouchPoint
   EM_BOOL onTarget;
   long targetX;
   long targetY;
+  // canvasX and canvasY are deprecated - there no longer exists a Module['canvas'] object, so canvasX/Y are no longer reported (register a listener on canvas directly to get canvas coordinates, or translate manually)
   long canvasX;
   long canvasY;
 } EmscriptenTouchPoint;
@@ -419,57 +426,10 @@ extern EMSCRIPTEN_RESULT emscripten_vibrate_pattern(int *msecsArray, int numEntr
 typedef const char *(*em_beforeunload_callback)(int eventType, const void *reserved, void *userData);
 extern EMSCRIPTEN_RESULT emscripten_set_beforeunload_callback_on_thread(void *userData, em_beforeunload_callback callback, pthread_t targetThread);
 
-typedef int EMSCRIPTEN_WEBGL_CONTEXT_HANDLE;
-
-typedef int EMSCRIPTEN_WEBGL_CONTEXT_PROXY_MODE;
-#define EMSCRIPTEN_WEBGL_CONTEXT_PROXY_DISALLOW 0
-#define EMSCRIPTEN_WEBGL_CONTEXT_PROXY_FALLBACK 1
-#define EMSCRIPTEN_WEBGL_CONTEXT_PROXY_ALWAYS   2
-
-typedef struct EmscriptenWebGLContextAttributes {
-  EM_BOOL alpha;
-  EM_BOOL depth;
-  EM_BOOL stencil;
-  EM_BOOL antialias;
-  EM_BOOL premultipliedAlpha;
-  EM_BOOL preserveDrawingBuffer;
-  EM_BOOL preferLowPowerToHighPerformance;
-  EM_BOOL failIfMajorPerformanceCaveat;
-
-  int majorVersion;
-  int minorVersion;
-
-  EM_BOOL enableExtensionsByDefault;
-  EM_BOOL explicitSwapControl;
-  EMSCRIPTEN_WEBGL_CONTEXT_PROXY_MODE proxyContextToMainThread;
-  EM_BOOL renderViaOffscreenBackBuffer;
-} EmscriptenWebGLContextAttributes;
-
-extern void emscripten_webgl_init_context_attributes(EmscriptenWebGLContextAttributes *attributes);
-
-extern EMSCRIPTEN_WEBGL_CONTEXT_HANDLE emscripten_webgl_create_context(const char *target, const EmscriptenWebGLContextAttributes *attributes);
-
-extern EMSCRIPTEN_RESULT emscripten_webgl_make_context_current(EMSCRIPTEN_WEBGL_CONTEXT_HANDLE context);
-
-extern EMSCRIPTEN_WEBGL_CONTEXT_HANDLE emscripten_webgl_get_current_context(void);
-
-extern EMSCRIPTEN_RESULT emscripten_webgl_get_drawing_buffer_size(EMSCRIPTEN_WEBGL_CONTEXT_HANDLE context, int *width, int *height);
-
-extern EMSCRIPTEN_RESULT emscripten_webgl_destroy_context(EMSCRIPTEN_WEBGL_CONTEXT_HANDLE context);
-
-extern EM_BOOL emscripten_webgl_enable_extension(EMSCRIPTEN_WEBGL_CONTEXT_HANDLE context, const char *extension);
-
-typedef EM_BOOL (*em_webgl_context_callback)(int eventType, const void *reserved, void *userData);
-extern EMSCRIPTEN_RESULT emscripten_set_webglcontextlost_callback_on_thread(const char *target, void *userData, EM_BOOL useCapture, em_webgl_context_callback callback, pthread_t targetThread);
-extern EMSCRIPTEN_RESULT emscripten_set_webglcontextrestored_callback_on_thread(const char *target, void *userData, EM_BOOL useCapture, em_webgl_context_callback callback, pthread_t targetThread);
-
-extern EM_BOOL emscripten_is_webgl_context_lost(const char *target);
-
-extern EMSCRIPTEN_RESULT emscripten_webgl_commit_frame(void);
-
-extern EM_BOOL emscripten_supports_offscreencanvas(void);
-
+// Sets the canvas.width & canvas.height properties.
 extern EMSCRIPTEN_RESULT emscripten_set_canvas_element_size(const char *target, int width, int height);
+
+// Returns the canvas.width & canvas.height properties.
 extern EMSCRIPTEN_RESULT emscripten_get_canvas_element_size(const char *target, int *width, int *height);
 
 extern EMSCRIPTEN_RESULT emscripten_set_element_css_size(const char *target, double width, double height);
@@ -515,8 +475,6 @@ extern void emscripten_html5_remove_all_event_listeners(void);
 #define emscripten_set_batterychargingchange_callback(userData, callback)                     emscripten_set_batterychargingchange_callback_on_thread(          (userData),               (callback), EM_CALLBACK_THREAD_CONTEXT_CALLING_THREAD)
 #define emscripten_set_batterylevelchange_callback(userData, callback)                        emscripten_set_batterylevelchange_callback_on_thread(             (userData),               (callback), EM_CALLBACK_THREAD_CONTEXT_CALLING_THREAD)
 #define emscripten_set_beforeunload_callback(userData, callback)                              emscripten_set_beforeunload_callback_on_thread(                   (userData),               (callback), EM_CALLBACK_THREAD_CONTEXT_MAIN_BROWSER_THREAD)
-#define emscripten_set_webglcontextlost_callback(target, userData, useCapture, callback)      emscripten_set_webglcontextlost_callback_on_thread(     (target), (userData), (useCapture), (callback), EM_CALLBACK_THREAD_CONTEXT_CALLING_THREAD)
-#define emscripten_set_webglcontextrestored_callback(target, userData, useCapture, callback)  emscripten_set_webglcontextrestored_callback_on_thread( (target), (userData), (useCapture), (callback), EM_CALLBACK_THREAD_CONTEXT_CALLING_THREAD)
 
 extern long emscripten_set_timeout(void (*cb)(void *userData), double msecs, void *userData);
 extern void emscripten_clear_timeout(long setTimeoutId);
@@ -543,8 +501,11 @@ extern void emscripten_console_error(const char *utf8String);
 extern void emscripten_throw_number(double number);
 extern void emscripten_throw_string(const char *utf8String);
 
+extern void emscripten_unwind_to_js_event_loop(void) __attribute__((noreturn));
+
 #ifdef __cplusplus
 } // ~extern "C"
 #endif
 
-#endif
+// For backwards compatibility, always include html5_webgl.h from html5.h
+#include <emscripten/html5_webgl.h>
